searchData={"content_type":"text/markdown","items":[{"doc":"A JSON library powered by Rust's Serde through a NIF","ref":"Jsonrs.html","title":"Jsonrs","type":"module"},{"doc":"Parses a JSON value from `input` string.","ref":"Jsonrs.html#decode/2","title":"Jsonrs.decode/2","type":"function"},{"doc":"iex> Jsonrs.decode(\"{\\\"x\\\":[1,2]}\")\n    {:ok, %{\"x\" => [1, 2]}}\n\n    iex> Jsonrs.decode(\"invalid\")\n    {:error, \"expected value at line 1 column 1\"}","ref":"Jsonrs.html#decode/2-examples","title":"Examples - Jsonrs.decode/2","type":"function"},{"doc":"Parses a JSON value from `input` string.\n\nSimilar to `decode/2` except it will raise in case of errors.","ref":"Jsonrs.html#decode!/2","title":"Jsonrs.decode!/2","type":"function"},{"doc":"iex> Jsonrs.decode!(\"{\\\"x\\\":[1,2]}\")\n    %{\"x\" => [1, 2]}\n\n    iex> Jsonrs.decode!(\"invalid\")\n    ** (ErlangError) Erlang error: \"expected value at line 1 column 1\"","ref":"Jsonrs.html#decode!/2-examples","title":"Examples - Jsonrs.decode!/2","type":"function"},{"doc":"Generates JSON corresponding to `input`.","ref":"Jsonrs.html#encode/2","title":"Jsonrs.encode/2","type":"function"},{"doc":"iex> Jsonrs.encode(%{\"x\" => [1,2]})\n    {:ok, \"{\\\"x\\\":[1,2]}\"}\n\n    iex> Jsonrs.encode(\"\\xFF\")\n    {:error, \"Expected to deserialize a UTF-8 stringable term\"}","ref":"Jsonrs.html#encode/2-examples","title":"Examples - Jsonrs.encode/2","type":"function"},{"doc":"Generates JSON corresponding to `input`.\n\nSimilar to `encode/1` except it will raise in case of errors.","ref":"Jsonrs.html#encode!/2","title":"Jsonrs.encode!/2","type":"function"},{"doc":"iex> Jsonrs.encode!(%{\"x\" => [1,2]})\n    \"{\\\"x\\\":[1,2]}\"\n\n    iex> Jsonrs.encode!(\"\\xFF\")\n    ** (ErlangError) Erlang error: \"Expected to deserialize a UTF-8 stringable term\"","ref":"Jsonrs.html#encode!/2-examples","title":"Examples - Jsonrs.encode!/2","type":"function"},{"doc":"Identical to `encode/1`. Exists to implement Phoenix interface and encodes to a single normal string.","ref":"Jsonrs.html#encode_to_iodata/2","title":"Jsonrs.encode_to_iodata/2","type":"function"},{"doc":"Identical to `encode!/1`. Exists to implement Phoenix interface and encodes to a single normal string.","ref":"Jsonrs.html#encode_to_iodata!/2","title":"Jsonrs.encode_to_iodata!/2","type":"function"},{"doc":"","ref":"Jsonrs.html#t:compression_algorithm/0","title":"Jsonrs.compression_algorithm/0","type":"type"},{"doc":"","ref":"Jsonrs.html#t:compression_level/0","title":"Jsonrs.compression_level/0","type":"type"},{"doc":"","ref":"Jsonrs.html#t:compression_options/0","title":"Jsonrs.compression_options/0","type":"type"},{"doc":"Protocol controlling how a value is encoded to JSON.\n\nBy default, structs are encoded as maps without the `:__struct__` key. If this\nis sufficient for your use, you don't need to implement this protocol.\n\nWhen implementing an encoding function, the goal is to turn your value into\nsomething already capable of being encoded.","ref":"Jsonrs.Encoder.html","title":"Jsonrs.Encoder","type":"protocol"},{"doc":"An implementation of this protocol for `Decimal` might look something like\n\n    defimpl Jsonrs.Encoder, for: Decimal do\n      def encode(value) do\n        Decimal.to_string(value)\n      end\n    end\n\nWhich will cause the Decimal to be encoded as its string representation","ref":"Jsonrs.Encoder.html#module-example","title":"Example - Jsonrs.Encoder","type":"protocol"},{"doc":"Converts `value` to a JSON-encodable type.","ref":"Jsonrs.Encoder.html#encode/1","title":"Jsonrs.Encoder.encode/1","type":"function"},{"doc":"","ref":"Jsonrs.Encoder.html#t:t/0","title":"Jsonrs.Encoder.t/0","type":"type"},{"doc":"# Jsonrs\n\nA safe NIF-based JSON library for Elixir powered by Rust's Serde. [API Documentation](https://hexdocs.pm/jsonrs/Jsonrs.html).","ref":"readme.html","title":"Jsonrs","type":"extras"},{"doc":"Add Jsonrs as a dependency in your `mix.exs` file.\n\n```elixir\ndef deps do\n  [{:jsonrs, \"~> 0.3\"}]\nend\n```\n\nTo avoid requiring an entire Rust toolchain for projects that use Jsonrs, pre-built binaries are provided via [Rustler Precompiled](https://github.com/philss/rustler_precompiled), which will download a platform-specific binary from the appropriate Github release during compilation. If you would like to avoid using the pre-built binaries, set the environment variable `FORCE_JSONRS_BUILD=true` when building.","ref":"readme.html#installation","title":"Installation - Jsonrs","type":"extras"},{"doc":"Jsonrs is much faster than other JSON libraries for Elixir, including other NIFs like jiffy.\n\nIt uses much less memory than other JSON libraries when operating in a mode with comparable features.\n\nIt has an API typical of Elixir JSON libraries. `decode/1` `encode/2` and `encode_to_iodata/2` are available along with their raising `!` counterparts, and structs can implement the `Jsonrs.Encoder` protocol to specify custom encoding.\n\nMultiple operating modes allow you to turn off features you don't need for extra performance. In the default 2-pass mode, Jsonrs works like Jason and Poison and honors custom encoding of structs. In the opt-in 1-pass (_lean_) mode, Jsonrs will ignore custom encoding impls and operate more like jiffy.","ref":"readme.html#summary","title":"Summary - Jsonrs","type":"extras"},{"doc":"Jsonrs is a 2-pass encoder by default.\n\nThe first pass involves mapping the input object according to any custom encodings implemented for types in the input. This is done in Elixir.\n\nThe second pass performs the actual encoding of the input to a JSON string. This is done in the Rust NIF.\n\nIf custom encoding definitions are not required, the first pass can be disabled entirely by passing `lean: true` in the encode opts. When operating in this mode, structs will always be encoded as if they were maps regardless of custom encoder implementations.","ref":"readme.html#encoder-design","title":"Encoder Design - Jsonrs","type":"extras"},{"doc":"Jsonrs has different performance characteristics in different modes.\nIn the default 2-pass mode, Jsonrs is comparable feature-wise to Jason or Poison and should always be much faster (~5-10x) and consume less memory (~3-30x) than either of them when working with any non-trivial JSON.\nIn lean mode, Jsonrs is comparable feature-wise to jiffy and should always be faster (~3x) and use less memory (~100x+) than it.\n\nFor example, here is a benchmark of Jsonrs's speed and memory consumption compared to other popular JSON libraries when encoding the 8MB [issue-90.json](https://github.com/devinus/poison/blob/a4208a6252f4e58fbcc8d9fd2f4f64c99e974cc8/bench/data/issue-90.json) from the Poison benchmark data.\n\n![Speed comparison of JSON libraries](https://raw.githubusercontent.com/benhaney/Jsonrs/bd8a008bcee93a0418646e4a8b32b8e26492fe97/bench-speed.png)\n\n![Memory comparison of JSON libraries](https://raw.githubusercontent.com/benhaney/Jsonrs/bd8a008bcee93a0418646e4a8b32b8e26492fe97/bench-memory.png)\n\nIt is easy to see that Jsonrs dramatically outperforms Poison and Jason, while Jsonrs-lean similarly outperforms jiffy.","ref":"readme.html#performance","title":"Performance - Jsonrs","type":"extras"},{"doc":"`Jsonrs.decode/1` takes a JSON string and turns it into an Elixir term.\n\n`Jsonrs.encode/2` takes an Elixir term and a keyword list of options and turns the input term into a JSON string with behavior defined by the options. Valid options are:\n* `lean`: disables the first encoder pass (default `false`)\n* `pretty`: turns on pretty printing when `true` or a non-negative-integer specifying indentation size (default `false`)\n* `compress`: turns on streaming compression when passed in.\n  * Currently you can pass `:gzip` to get default level gzip compression\n  * You can also pass `{:gzip, 0..9}` to get a specific level of gzip compression\n  * Other compression forms may be accepted in the future","ref":"readme.html#quick-start","title":"Quick Start - Jsonrs","type":"extras"},{"doc":"#","ref":"readme.html#examples","title":"Examples - Jsonrs","type":"extras"},{"doc":"```elixir\niex> Jsonrs.encode!(%{\"x\" => [1, 2], \"y\" => 0.5})\n\"{\\\"x\\\":[1,2],\\\"y\\\":0.5}\"\n\niex> Jsonrs.decode!(\"{\\\"x\\\":[1,2],\\\"y\\\":0.5}\")\n%{\"x\" => [1, 2], \"y\" => 0.5}\n\niex> Jsonrs.encode!(%{\"x\" => false, \"y\" => []}, pretty: true) |> IO.puts()\n{\n  \"x\": false,\n  \"y\": []\n}\n```\n\n#","ref":"readme.html#basic-usage","title":"Basic usage - Jsonrs","type":"extras"},{"doc":"Implement `Jsonrs.Encoder` for your struct, with an `encode/1` function that returns any other type encodable by Jsonrs (it does not need to return a string).\n\nFor example:\n\n```elixir\ndefimpl Jsonrs.Encoder, for: [MapSet, Range, Stream] do\n  def encode(struct), do: Enum.to_list(struct)\nend\n```","ref":"readme.html#defining-a-custom-encoder","title":"Defining a custom encoder - Jsonrs","type":"extras"},{"doc":"Inherited from `serde_json`, decoding is limited to a nesting depth of 128. As per RFC 7159, section 9:\n> An implementation may set limits on the maximum depth of nesting.","ref":"readme.html#limitations","title":"Limitations - Jsonrs","type":"extras"},{"doc":"Jsonrs is built on the backs of giants. It is connecting the Rust libraries [Serde_rustler](https://github.com/sunny-g/serde_rustler) and [Serde_json](https://github.com/serde-rs/json), exposing them to Elixir through [Rustler](https://github.com/rusterlium/rustler), and wrapping the NIF in an interface inspired by [Jason](https://github.com/michalmuskala/jason). Without these projects, Jsonrs probably wouldn't exist.","ref":"readme.html#credits","title":"Credits - Jsonrs","type":"extras"}]}